/*
 * LCD.c
 *
 * Created: 23.4.2013 13:07:37
 *  Author: mjankovec
 */ 

#include "LCD.h"

FILE LCDstream = FDEV_SETUP_STREAM(LCD_putc, NULL, _FDEV_SETUP_WRITE); 

char row,col;

int LCD_putc (char c, FILE *stream)
{
	switch (c)
	{
		case '\r':	LCD_gotoxy(0,row); break;
		case '\n':	LCD_gotoxy(col,1-row); break;
		default:	if (col > 15) LCD_gotoxy(0,1-row);
					LCD_Write(Data,c);
					col++;
					break;
	}
	return 0;
}


//************************************
// Method:    LCD_Busy
// FullName:  LCD_Busy
// Access:    public 
// Returns:   char
// Qualifier: Funkcija vrne 0, ?e je LCD pripravljen za vpis podatka
//************************************

char LCD_Busy()
{
	char data;

	//D4 - D7 = PORTA 4-7
	//RS(PC7) = 0, RW(PC6) = 1, E(PC0)
	LCD_IR();
	LCD_R();
	
	//Tu beremo zgornje 4 bite
	LCD_E_set();
	asm("nop");
	asm("nop");
	data = LCD_DataPin; //preberi zgornje stiri bite
	LCD_E_clr();
	asm("nop");
	asm("nop");
	
	//Tu beremo spodnje 4 bite (samo da jih v bistvu ignoriramo)
	LCD_E_set();
	asm("nop");
	asm("nop");
	LCD_E_clr();
	
	if (data & 0x80)	return 1;	// pomeni da je zaseden
	else return 0;	// pomeni da ni zaseden
}



//************************************
// Method:    LCD_BusyWait
// FullName:  LCD_BusyWait
// Access:    public 
// Returns:   void
// Qualifier: funkcija ?aka, dokler ni LCD pripravljen ali dokler ne pote?e timeout. Timeout je odvisen od ure procesorja.
// Parameter: unsigned int timeout
//************************************
void LCD_BusyWait(unsigned int timeout)
{
	for (unsigned int i=0;i<timeout;i++)
	{
		if (!LCD_Busy()) break;
	}	
}

//************************************
// Method:    LCD_Write
// FullName:  LCD_Write
// Access:    public 
// Returns:   void
// Qualifier: Funkcija vpie podatek v LCD
// Parameter: enum LCD_Register reg
// Parameter: char data
//************************************
void LCD_Write(enum LCD_Register reg, char data)
{
	char temp;
	
	LCD_BusyWait(1000);
	
	//D4 - D7 = PORTA 4-7

	if (reg == Instr) LCD_IR(); // pii v intrukcijski register
	else LCD_DR(); // pii v podatkovni register
	
	LCD_W();
	LCD_E_set();
	
	// write high nibble
	temp = data>>4;	 // dddd 0000 -> 0000 dddd
	LCD_Write_Nibble(temp);

	LCD_E_clr();
	asm("nop");
	asm("nop");
	
	LCD_E_set();
	
	// write low nibble
	temp = data & 0x0F;	 //  0000 dddd
	LCD_Write_Nibble(temp);
	
	LCD_E_clr();
}

//************************************
// Method:    LCD_Init
// FullName:  LCD_Init
// Access:    public 
// Returns:   void
// Qualifier: Funkcija inicializira LCD
//************************************
void LCD_Init()
{
	
	//nastavi ustrezne pine kot izhode
	LCD_E_Dir |= LCD_E_Mask;
	LCD_RS_Dir |= LCD_RS_Mask;
	LCD_RW_Dir |= LCD_RW_Mask;
	LCD_DataDir |= LCD_DataMask;
	LCD_Backlight_Dir |= LCD_Backlight_Mask;
	
	LCD_Backlight_ON();
	
	// tukaj vpiite startup sekvenco za incializaicjo LCD (slide 17)
	LCD_IR();
	LCD_W();

	_delay_ms(20);

	LCD_E_set();
	LCD_Write_Nibble(0x03);	// interface 8-bit
	LCD_E_clr();
	
	_delay_ms(5);

	LCD_E_set();
	LCD_Write_Nibble(0x03);	// interface 8-bit
	LCD_E_clr();

	_delay_us(120);
	
	LCD_E_set();
	LCD_Write_Nibble(0x03);	// interface 8-bit
	LCD_E_clr();

	_delay_us(40);
	
	LCD_E_set();
	LCD_Write_Nibble(0x02);	// interface 4-bit
	LCD_E_clr();

	LCD_Write(Instr,0x28);	//001DL NF00 DL=0(interface 4-bit), N=1(2lines) F=0(char size 5x8)
	LCD_Write(Instr,0x08);	//0000 1DCB Display OFF
	LCD_Write(Instr,0x01);	//0000 0001 Clear Display
	_delay_ms(2);
	LCD_Write(Instr,0x06);	//0000 01IS I/D=1 (Levo Proti Desni), S=0(No display shift)
	LCD_control(LCD_ON | LCD_CURSOR_ON | LCD_BLINK_ON);
}

//************************************
// Method:    LCD_gotoxy
// FullName:  LCD_gotoxy
// Access:    public
// Returns:   void
// Qualifier: Funkcija nastavi pozicijo kurzorja
// Parameter: char col
// Parameter: char row
//************************************
void LCD_gotoxy(char _col, char _row)
{
	// column: 0 or 1, row: 0 to 39
	if (_col > 39) _col = 39;
	if (_row > 1 ) _row = 1;
	if (_row == 0) LCD_Write(Instr, 0x80 | 0x00 | _col);
	else		  LCD_Write(Instr, 0x80 | 0x40 | _col);
	row=_row;
	col=_col;
}


//************************************
// Method:    LCD_shiftLeft
// FullName:  LCD_shiftLeft
// Access:    public
// Returns:   void
// Qualifier: Funkcija premakne vsebino ekrana proti levi
// Parameter: char cnt
//************************************
void LCD_shiftLeft(int cnt)
{
	for (int i=0; i<cnt; i++) LCD_Write(Instr, 0x18);
}

//************************************
// Method:    LCD_shiftRight
// FullName:  LCD_shiftRight
// Access:    public
// Returns:   void
// Qualifier: Funkcija premakne vsebino ekrana proti desni
// Parameter: char cnt
//************************************
void LCD_shiftRight(int cnt)
{
	for (int i=0; i<cnt; i++) LCD_Write(Instr, 0x1C);
}

//************************************
// Method:    LCD_WriteString
// FullName:  LCD_WriteString
// Access:    public
// Returns:   void
// Qualifier: Funkcija izpie znakovni niz z naslova *str
//				pise do konca spomina (dolgi 40 znakov), ne do konca displaya
// Parameter: char *str
//************************************
void LCD_WriteString(char *str)
{
	int i=0;
	while (str[i])
	{
		LCD_Write(Data,str[i]);
		i++;
		col++;
		// memory is 40 bytes long, then it resumes in the next line
		if (col>39) {row=1-row; col=0;}
	}
}

//************************************
// Method:    LCD_WriteBytes
// FullName:  LCD_WriteBytes
// Access:    public
// Returns:   void
// Qualifier: Funkcija izpie count znakov, ki se zanejo na naslovu *data
//				pise do konca spomina (dolgi 40 znakov), ne do konca displaya
// Parameter: char *data
// Parameter: char count
//************************************
void LCD_WriteBytes(char *data, char count)
{
	int i=0;
	while (i<count)
	{
		LCD_Write(Data,data[i]);
		i++;
		col++;
		// memory is 40 bytes long, then it resumes in the next line
		if (col>39) {row=1-row; col=0;}
	}
}
