/*
 * LCD.h
 *
 * Created: 23.4.2013 22:14:06
 *  Author: mjankovec
 */ 


#ifndef LCD_H_
#define LCD_H_

#include <avr/io.h>			//knjiznica z definicijami registrov
#define F_CPU 18432000UL	// dolocitev frekvence oscilatorja za funkcije iz delay.h - mora biti pred #include <util/delay.h>
#include <util/delay.h>		//knjiznica s funkcijami delay
#include <stdio.h>

//Definicije prikljucitve LCD - spreminja uporabnik

#define LCD_DataPort	PORTA
#define LCD_DataPin		PINA
#define LCD_DataDir		DDRA
#define LCD_DataMask	0xf0 //1111 0000
#define LCD_Shift		4	 //0000 DDDD -> DDDD 0000

#define LCD_E_Port	PORTC
#define LCD_E_Dir	DDRC
#define LCD_E_Mask	0x01

#define LCD_RW_Port	PORTC
#define LCD_RW_Dir	DDRC
#define LCD_RW_Mask	0x40

#define LCD_RS_Port	PORTC
#define LCD_RS_Dir	DDRC
#define LCD_RS_Mask	0x80

#define LCD_Backlight_Port	PORTD
#define LCD_Backlight_Dir	DDRD
#define LCD_Backlight_Mask	0x80


// definicije kontrolnih signalov

#define LCD_E_set()	LCD_E_Port |= LCD_E_Mask
#define LCD_E_clr()	LCD_E_Port &= ~LCD_E_Mask

#define LCD_R()	LCD_RW_Port |= LCD_RW_Mask; LCD_DataDir &= ~LCD_DataMask
#define LCD_W()	LCD_RW_Port &= ~LCD_RW_Mask; LCD_DataDir |= LCD_DataMask

#define LCD_DR()	LCD_RS_Port |= LCD_RS_Mask
#define LCD_IR()	LCD_RS_Port &= ~LCD_RS_Mask

#define LCD_Backlight_ON()	LCD_Backlight_Port |= LCD_Backlight_Mask;	//prizgi osvetlitev
#define LCD_Backlight_OFF()	LCD_Backlight_Port &= ~LCD_Backlight_Mask;	//prizgi osvetlitev

// definicije pisanja na podatkovno vodilo // 0000 xxxx    // dddd xxxx
#define LCD_Write_Nibble(x)	LCD_DataPort &= ~LCD_DataMask; LCD_DataPort |= (x<<LCD_Shift)

//definicija enum podatkovnega tipa LCD_register
enum LCD_Register {Data=0,Instr=1};

#define LCD_clrscr() LCD_Write(Instr,0x01); row=0; col=0; LCD_BusyWait(3000)
#define LCD_ON	0x04
#define LCD_OFF 0
#define LCD_CURSOR_ON	0x02
#define LCD_CURSOR_OFF	0
#define LCD_BLINK_ON	0x01
#define LCD_BLINK_OFF	0
#define LCD_control(x) LCD_Write(Instr,((x) & 0x07) | 0x08)

FILE LCDstream;
char row,col;

int LCD_putc (char c, FILE *stream);
char LCD_Busy();
void LCD_BusyWait(unsigned int timeout);
void LCD_Write(enum LCD_Register reg, char data);
void LCD_Init();
void LCD_gotoxy(char _col, char _row);
void LCD_WriteString(char *str);
void LCD_WriteBytes(char *data, char count);
void LCD_shiftLeft(int cnt);
void LCD_shiftRight(int cnt);

#endif /* LCD_H_ */